@@ -1,6 +1,6 @@
 package Dist::Zilla::Role::MetaCPANInterfacer;
 
-our $VERSION = '0.93'; # VERSION
+our $VERSION = '0.94'; # VERSION
 # ABSTRACT: something that will interact with MetaCPAN's API
 
 use sanity;
@@ -11,9 +11,14 @@ use WWW::Mechanize::Cached::GZip;
 use HTTP::Tiny::Mech;
 use MetaCPAN::API;
 
+use POSIX ();
+use File::Temp ();  # 'tmpnam' defined both here and POSIX
 use Path::Class;
 use File::HomeDir;
 use Scalar::Util qw{blessed};
+use List::AllUtils qw{min};
+
+use namespace::clean;
 
 has mcpan => (
    is      => 'rw',
@@ -49,11 +54,18 @@ has mcpan_cache => (
    isa     => 'Object',
    lazy    => 1,
    default => sub {
+      # don't use $HOME if we are in the middle of testing
+      my $home_dir = $ENV{HARNESS_ACTIVE} ? File::Temp->newdir() : File::HomeDir->my_home;
+      my $root_dir = dir($home_dir)->subdir('.dzil', '.webcache');
       CHI->new(
          namespace  => 'MetaCPAN',
          driver     => 'File',
          expires_in => '1d',
-         root_dir   => Path::Class::dir( File::HomeDir->my_home )->subdir('.dzil', '.webcache')->stringify,
+         root_dir   => $root_dir->stringify,
+
+         # https://rt.cpan.org/Ticket/Display.html?id=78590
+         on_set_error   => 'die',
+         max_key_length => min( POSIX::PATH_MAX - length( $root_dir->subdir('MetaCPAN', 0, 0)->absolute->stringify ) - 4 - 8, 248),
       )
    },
 );
@@ -62,11 +74,10 @@ sub _mcpan_set_agent_str {
    my ($self, $ua) = @_;
    my $o = ucfirst($^O);
    
-   use POSIX;
-   my ($sysname, $nodename, $release, $version, $machine) = POSIX::uname();
+   my ($sysname, $nodename, $release, $version, $machine) = POSIX::uname;
    my $os = join('; ', "$sysname $release", $machine, $version);
    
-   my $v = $self->VERSION;
+   my $v = $self->VERSION || '';
    $ua->agent("Mozilla/5.0 ($o; $os) ".blessed($self)."/$v ".$ua->_agent);
 
    return $ua;
